/* GSI - general sound interface
 *
 * gsi_patchload_opl3.c
 *
 * Copyright (C) W.H.Scholten 1998
 *
 */

/* This software is licensed under the CGPL version 0.4 or later
 * (see the file LICENSE.CGPL for details)
 */


/*__________________INCLUDES___________________*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>

#include <gsi/mytypes.h> /* use gsi_int16 etc instead ? */
#include <gsi/config.h>
#include <gsi/debug.h>
#include "midi.h"

#ifdef __linux__
#  include <sys/soundcard.h>
#endif
#ifdef __FreeBSD__
#  include <machine/soundcard.h>
#endif

#ifdef USE_OSS /* This code is currrently OSS only */

#ifdef USE_OPL3_SYNTH


/*__________________DEFINITIES___________________*/
/* Defaults: */
#define OPL3_PATCH_DIR "/etc/"
#define OPL3_INSTR_PATCH_FILE "std.o3"
#define OPL3_DRUMS_PATCH_FILE "drums.o3"

#define OPL2_PATCH_DIR "/etc/"
#define OPL2_INSTR_PATCH_FILE "std.sb"
#define OPL2_DRUMS_PATCH_FILE "drums.sb"


/*__________________EXTERN___________________*/
SEQ_USE_EXTBUF();


/*__________________VARIABELEN___________________*/

char *opl3_patch_dir=OPL3_PATCH_DIR;
char *opl3_instr_patch_file=OPL3_INSTR_PATCH_FILE;
char *opl3_drums_patch_file=OPL3_DRUMS_PATCH_FILE;

char opl3_instr_file[256];
char opl3_drums_file[256];

static int instrmap[256] = {-1, 0};
static int initialized = 0;

int	 drum_fd = -1, std_fd = -1; /* global, we keep them opened unless special actions are done (loading a patch from memory) */

int  use_opl3; /* switch to opl3 mode ? */


/*__________________PROGRAMMA___________________*/
int opl3_init(int seq_fd, int dev)
{

	DEB1(fprt(stderr, "__GSI: opl3_init\n"));
	if (!initialized) {
		strcpy(opl3_instr_file, opl3_patch_dir);
		if (opl3_instr_patch_file[0]=='/') /* absolute path */
			strcpy(opl3_instr_file, opl3_instr_patch_file);
		else
			strcat(opl3_instr_file, opl3_instr_patch_file);

		strcpy(opl3_drums_file, opl3_patch_dir);
		if (opl3_drums_patch_file[0]=='/') /* absolute path */
			strcpy(opl3_drums_file, opl3_drums_patch_file);
		else
			strcat(opl3_drums_file, opl3_drums_patch_file);

		if ((std_fd=open(opl3_instr_file, O_RDONLY, 0))==-1) {
			perror(opl3_instr_file);
		}
		if ((drum_fd=open(opl3_drums_file, O_RDONLY, 0))==-1) {
			perror(opl3_drums_file);
		}
	}
  	initialized=1;

	if (voices[dev] != NULL) free(voices[dev]);
	voices[dev] = malloc(sizeof(struct voice_state) * card_info[dev].nr_voices);


#if 0 /* Not for opl3 ? */
	if (ioctl (seq_fd, SNDCTL_SEQ_RESETSAMPLES, &dev) == -1)
		{
			fprintf(stderr, "GSI: Can't reset GM programs of device %d\n", dev);
			perror("OPL3 init");
			return -1;
		}
#endif

	use_opl3=True; /* how determine if we have OPL3 and if we want it? */


	return 0;
}

int opl3_load (int seq_fd, int type, int dev, int pgm)
{
	int  i;
	unsigned char buf[256];
	struct sbi_instrument patch;

	char *name;

	DEB1(fprt(stderr, "__GSI: opl3_load\n"));
	if (!initialized)
		opl3_init(seq_fd, dev);

	if (std_fd == -1 || drum_fd == -1)
		return -1;

	if (pgm<0 || pgm>255)
		return 0; /* ignore out of range values */

	if (instrmap[pgm] == pgm)
		return 0; /* already loaded */

	instrmap[pgm] = pgm;

	patch.key = OPL3_PATCH;
	patch.device  = dev;
	patch.channel = pgm;

	if (pgm < 128) { /* melodic patches */
		name = opl3_instr_file;
		if (lseek(std_fd, pgm*60, SEEK_SET)==-1) {
			perror(opl3_instr_file);
			return -1;
		}
		if (read(std_fd, buf, 60) != 60) {
			perror(opl3_instr_file);
			return -1;
		}
	}
	else { /* drum patches */
		name = opl3_drums_file;
		if (lseek(drum_fd, (pgm-128)*60, SEEK_SET)==-1) {
			perror(opl3_drums_file);
			return -1;
		}
		if (read(drum_fd, buf, 60) != 60) {
			perror(opl3_drums_file);
			return -1;
		}
	}

	if (strncmp(buf, "4OP", 3) != 0)
		if (strncmp(buf, "2OP", 3) == 0) {
			patch.key = FM_PATCH;
		}
		else {
			fprintf(stderr, "GSI: Invalid OPL3 patch file %s, instr=%d\n", name, pgm & 127);
			return 0;
		}

	for (i=0; i<22; i++)
		patch.operators[i] = buf[i+36];

	/* send it off to the sound card with OSS's functions */
#define seqfd seq_fd
	SEQ_WRPATCH (&patch, sizeof (patch));
#undef seqfd
	return 0;
}

#endif /* USE_OPL3_SYNTH */

#endif /* USE_OSS */

/*---------------------------------------------------------------------------*
 *	EOF gsi_patchload_opl3.c
 *---------------------------------------------------------------------------*/
