/* GSI - general sound interface
 * gsitest: tests the gsi sound server
 *
 * main.cc
 *
 * By W.H.Scholten 1998
 *
 * License for this file: Public domain, use as a basis for your own programs
 */


/* Don't complain about lack of docs on this test program!
   It's my test and it changes a lot ..

   Shift + 1 - 8: set dsp to:
       mono/stereo/8 bit mulaw/8 bit unsigned/16bit/8000 Hz/16000 Hz/22050 Hz

   1 - 0 = play slot  0 -  9
   z=position sound at (x,z,y)=(-64,-64,0) [ left ]
   x=position sound at (x,z,y)=(  0,-64,0) [ center ]
   c=position sound at (x,z,y)=(+64,-64,0) [ right ]

   , = move sound to the left
   . = move sound to the right


   b load song InstantX.mid
   n start song
   m stop song
   k continue song
   */

/*__________________INCLUDES___________________*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>

#include <math.h>

#include <gsi/gsi.h>

#define USE_CURSES
#ifdef USE_CURSES
#  include <curses.h>
#endif

/*__________________VARIABELEN___________________*/
int  snd_channels=2;
int  frag_size=8;
//int  snd_type=GSI_8BIT_MULAW;
int  snd_type=GSI_8BIT;
int  dsp_speed=8000;
//#  define SOUNDDIR "/home/wouter/projecten/gsi/test/sounds"
#  define SOUNDDIR "./sounds"
#  define SOUNDFILE "test.sounds"


/*__________________PROGRAMMA___________________*/
int main(int argc, char *argv[]) {
	int  end;
	char ch;
	int  x, dx, dopplerspeed;

#ifdef USE_CURSES
	// see man ncurses for more info on these functions
	initscr();
	cbreak();
	//noecho();

	//nonl(); /* no newline */
	intrflush(stdscr, FALSE);
	keypad(stdscr, TRUE);
#endif

	frag_size = 8+snd_channels-1+(snd_type+1)/2-1;
	gsi_init();
	gsi_init_pcm(dsp_speed, snd_channels, snd_type, frag_size, SOUNDDIR, SOUNDFILE);
	gsi_send_commands(1, SND_CALLBACK_ON);

	{
#define TEST2
#ifdef TEST1
		char *fdata;
		int  size;
		FILE *fp; fp=fopen("../music/110.au", "r");

		fseek(fp, 0, SEEK_END);
		size=ftell(fp);
		fdata=(char *) malloc(size);
		fseek(fp, 0, SEEK_SET);
		fread(fdata, size, 1, fp);
		// should skip the au offset here!!, oh well, it's just a test and
		// will play a bit of junk before the real data begins.
		gsi_send_sample(19, 11025, GSI_8BIT/*type*/, 1/*channels*/, size/*size*/, fdata/*data*/);

		free(fdata);
		fclose(fp);
#endif
#ifdef TEST2
		gsi_load_file(19, "../music/110.au");
#endif

		gsi_load_file(18, "marvin.au");
	}

	end=0;

	do {
		ch=getchar();
		switch (ch) {
		case  27:  end=1; break; /* Escape */
		case 'q':  gsi_kill_sounds(); break;

			/* Shift 1- shift 8 */
		case '!' : /* Shift-1 mono DSP */
			gsi_send_commands(3, SND_SET_DSP_CHANNELS, 1, SND_GRAB_DSP); break;
		case '@' : /* Shift-2 stereo DSP */
			gsi_send_commands(3, SND_SET_DSP_CHANNELS, 2, SND_GRAB_DSP); break;
		case '#' : /* Shift-3 8 bit mulaw DSP */
			gsi_send_commands(3, SND_SET_DSP_BYTES, GSI_8BIT_MULAW, SND_GRAB_DSP); break;
		case '$' : /* Shift-4 8 bit DSP */
			gsi_send_commands(3, SND_SET_DSP_BYTES, GSI_8BIT, SND_GRAB_DSP); break;
		case '%' : /* Shift-5 16 bit DSP */
			gsi_send_commands(3, SND_SET_DSP_BYTES, GSI_16BIT, SND_GRAB_DSP); break;
		case '^' : /* Shift-6 8000 Hz bit DSP */
			gsi_send_commands(3, SND_SET_DSP_SPEED, 8000>>8, 8000 &0xff, SND_GRAB_DSP); break;
		case '&' : /* Shift-7 16000 Hz bit DSP */
			gsi_send_commands(3, SND_SET_DSP_SPEED, 16000>>8, 16000 &0xff, SND_GRAB_DSP); break;
		case '*' : /* Shift-8 22050 Hz bit DSP */
			gsi_send_commands(3, SND_SET_DSP_SPEED, 22050>>8, 22050 &0xff, SND_GRAB_DSP); break;

		case '1' : gsi_play_sound_flush(0,0); break;
		case '2' : gsi_play_sound_flush(1,0); break;
		case '3' : gsi_play_sound_flush(2,0); break;
		case '4' : gsi_play_sound_flush(3,0); break;
		case '5' : gsi_play_sound_flush(4,0); break;
		case '6' : gsi_play_sound_flush(5,0); break;
		case '7' : gsi_play_sound_flush(6,0); break;
		case '8' : gsi_play_sound_flush(7,0); break;

		case '9' : gsi_play_sound_flush(8,0); break; /* marvin.au */
		case '(' : gsi_play_sound_flush(18,0); break;

		case '0' : gsi_play_sound_flush(9,0); break; /* man_down.au */
		case ')' : gsi_play_sound_flush(19,0); break;

		case 'a' : gsi_play_sound_flush(9, SND_REPEAT); break;
		case 's' : gsi_play_sound_flush(9, SND_KILL); break;

		case 'z' : x=-64; dx=0; //left
			gsi_send_commands(6, SND_SET_X | SND_CURRENT, x >>8, x &255, SND_SET_DX | SND_CURRENT, dx >>8, dx &255); break;
		case 'x' : x=  0; dx=0; //middle
			gsi_send_commands(6, SND_SET_X | SND_CURRENT, x >>8, x &255, SND_SET_DX | SND_CURRENT, dx >>8, dx &255); break;
		case 'c' : x= 64; dx=0; //right
			gsi_send_commands(6, SND_SET_X | SND_CURRENT, x >>8, x &255, SND_SET_DX | SND_CURRENT, dx >>8, dx &255); break;
		case ',' : dx=-80;  //to left
			gsi_send_commands(3, SND_SET_DX | SND_CURRENT, dx >>8, dx &255); break;
		case '.' : dx= 80;  //to right
			gsi_send_commands(3, SND_SET_DX | SND_CURRENT, dx >>8, dx &255); break;

		case '<' : dx=-256;  //to left
			dopplerspeed=20;
			gsi_send_commands(4, SND_SET_DX | SND_CURRENT, dx >>8, dx &255, SND_DOPPLER_ON | SND_CURRENT); break;

		case '>' : dx= 256;  //to right
			dopplerspeed=20;
			gsi_send_commands(4, SND_SET_DX | SND_CURRENT, dx >>8, dx &255, SND_DOPPLER_ON | SND_CURRENT); break;


		case ';' : dx=-5;  //to left
			gsi_send_commands(3, SND_SET_CHANNEL_DX | SND_CURRENT, dx >>8, dx &255); break;
		case '\'': dx= 5; //to right
			gsi_send_commands(3, SND_SET_CHANNEL_DX | SND_CURRENT, dx >>8, dx &255); break;
		case '[' : dx=-5;  //to left
			gsi_send_commands(4, SND_SET_CHANNEL_DX, 5, dx >>8, dx &255); break;
		case ']' : dx= 5; //to right
			gsi_send_commands(4, SND_SET_CHANNEL_DX, 5, dx >>8, dx &255); break;


		case 'F': gsi_send_commands(2, SND_FREE_SOUND, 0);
			break;
		case 'A': gsi_send_commands(1, SND_AUTO_DSP);
			break;
		case 'G': gsi_send_commands(1, SND_GRAB_DSP);
			break;
		case 'R': gsi_send_commands(1, SND_RELEASE_DSP);
			break;
		case 'V': gsi_play_channel_sound(0, 19, SND_PLAY); break;
		case 'v': gsi_kill_channel_sound(0); break;

		case 'D': gsi_play_channel_sound(1, 0, SND_PLAY); break;
		case 'd': gsi_kill_channel_sound(1); break;

		case 'E': gsi_play_channel_sound(5, 6, SND_PLAY); break;
		case 'e': gsi_kill_channel_sound(5); break;

		case 'o': gsi_send_commands(4, SND_SET_CHANNEL_ANGLE | SND_CURRENT, 64, SND_SET_CHANNEL_VOLUME | SND_CURRENT, 220); break;

		case 'p' : gsi_send_commands(2, SND_SET_CHANNEL_VOLUME | SND_CURRENT, 100); break;
		case 'U' : gsi_send_commands(2, SND_SET_CHANNEL_ANGLE | SND_CURRENT, 192); break;
		case 'I' : gsi_send_commands(2, SND_SET_CHANNEL_ANGLE | SND_CURRENT, 128); break;
		case 'O' : gsi_send_commands(2, SND_SET_CHANNEL_ANGLE | SND_CURRENT, 64); break;
		case 'P' : gsi_send_commands(2, SND_SET_CHANNEL_ANGLE | SND_CURRENT, 0); break;

			/* Music, midi */
		case 'b': gsi_load_song_file("./music/InstantX.mid");
			break;
		case 'n': gsi_play_song(1);
			break;
		case 'm': gsi_stop_song();
			break;
		case 'k': gsi_continue_song();
			break;
		default  : break;
		}
		gsi_flush();
	} while (!end);
#ifdef USE_CURSES
	nocbreak(); /* buffer IO again */
	endwin();
#endif
	gsi_close();
	{ int status; wait(&status); } /* only useful for the pipe version */
	printf("\nProgram terminated\n");

	return 0;
}

